import type { evaluate, isTopType } from "./generics.js";
export declare const hasDomain: <data, domain extends Domain>(data: data, domain: domain) => data is data & inferDomain<domain>;
type DomainTypes = {
    bigint: bigint;
    boolean: boolean;
    number: number;
    object: object;
    string: string;
    symbol: symbol;
    undefined: undefined;
    null: null;
};
export type inferDomain<domain extends Domain> = Domain extends domain ? unknown : DomainTypes[domain];
export type Domain = evaluate<keyof DomainTypes>;
export type NullishDomain = "undefined" | "null";
export type NonNullishDomain = Exclude<Domain, NullishDomain>;
export type PrimitiveDomain = Exclude<Domain, "object">;
export type Primitive = inferDomain<PrimitiveDomain>;
export type domainOf<data> = isTopType<data> extends true ? Domain : data extends object ? "object" : data extends string ? "string" : data extends number ? "number" : data extends boolean ? "boolean" : data extends undefined ? "undefined" : data extends null ? "null" : data extends bigint ? "bigint" : data extends symbol ? "symbol" : never;
export declare const domainOf: <data>(data: data) => domainOf<data>;
/** Each domain's completion for the phrase "Must be _____" */
export declare const domainDescriptions: {
    readonly bigint: "a bigint";
    readonly boolean: "boolean";
    readonly null: "null";
    readonly number: "a number";
    readonly object: "an object";
    readonly string: "a string";
    readonly symbol: "a symbol";
    readonly undefined: "undefined";
};
export {};
